@@ -15,9 +15,8 @@ class MarpaX::Languages::M4::Impl::Parser {
     use Marpa::R2;                               # 2.103_004;
     use Scalar::Util qw/readonly/;
     use Types::Common::Numeric -all;
-    use Throwable::Factory NoLexeme => undef, NoParseValue => undef;
 
-    our $VERSION = '0.006'; # VERSION
+    our $VERSION = '0.007'; # VERSION
 
     our $AUTHORITY = 'cpan:JDDPAUSE'; # AUTHORITY
 
@@ -131,10 +130,11 @@ COMMA ~ ',' _WS_any
 
         my $maxPos = length( ${$inputRef} ) - 1;
         #
-        # Protect the case of empty string
+        # Protect the case of empty string.
+        # This is affecting all parsing levels.
         #
         if ( $pos > $maxPos ) {
-            return undef;
+            return;
         }
         #
         # Get the lexemes ordering
@@ -217,12 +217,21 @@ COMMA ~ ',' _WS_any
 
             my $canCollectArguments = true;
             if ( $g == $BYMACROARGUMENTS_G ) {
-                $isQuotedString
-                    = $self->parser_isQuotedstring( ${$inputRef}, $rc{pos},
-                    $maxPos, \$QuotedstringValue, \$QuotedstringLength );
-                $isComment = $self->parser_isComment( ${$inputRef}, $rc{pos},
-                    $maxPos, \$CommentValue, \$CommentLength );
-                $canCollectArguments = !$isQuotedString && !$isComment;
+                my $blockOk = false;
+                try {
+                    $isQuotedString
+                        = $self->parser_isQuotedstring( ${$inputRef},
+                        $rc{pos},
+                        $maxPos, \$QuotedstringValue, \$QuotedstringLength );
+                    $isComment
+                        = $self->parser_isComment( ${$inputRef}, $rc{pos},
+                        $maxPos, \$CommentValue, \$CommentLength );
+                    $canCollectArguments = !$isQuotedString && !$isComment;
+                    $blockOk = true;
+                };
+                if ( !$blockOk ) {
+                    goto return_error;
+                }
             }
 
             foreach (@lexemeNames) {
@@ -304,14 +313,18 @@ COMMA ~ ',' _WS_any
                         # in the context of macro arguments grammar
                         # Ditto for COMMENT.
                         #
-                        my $method = 'parser_is' . ucfirst( lc($_) );
-                        if ($self->$method(
-                                ${$inputRef}, $rc{pos},
-                                $maxPos,      \$lexemeValue,
-                                \$lexemeLength
-                            )
-                            )
-                        {
+                        my $method  = 'parser_is' . ucfirst( lc($_) );
+                        my $blockOk = false;
+                        my $isToken;
+                        try {
+                            $isToken = $self->$method( ${$inputRef}, $rc{pos},
+                                $maxPos, \$lexemeValue, \$lexemeLength );
+                            $blockOk = true;
+                        };
+                        if ( !$blockOk ) {
+                            goto return_error;
+                        }
+                        if ($isToken) {
                             $lexeme = $_;
                             last;
                         }
@@ -322,9 +335,7 @@ COMMA ~ ',' _WS_any
             # Nothing ?
             #
             if ( Undef->check($lexeme) ) {
-                NoLexeme->throw(
-                    "No token found at position $rc{pos} (20 first characters): "
-                        . substr( ${$inputRef}, $rc{pos}, 20 ) );
+                goto return_error;
             }
             #
             # If it is a word, check if this is eventually a macro
@@ -339,93 +350,32 @@ COMMA ~ ',' _WS_any
                     )
                     )
                 {
+                    #
+                    # Collect macro arguments
+                    #
+                    my @args = ();
 
-                    my $macroName = $lexemeValue;
-                    my $printableMacroName
-                        = $self->_printable( $macroName, true );
+                    if ( $lparenPos >= 0 ) {
 
-                    # $self->logger_debug(
-                    #     '[%d..%d/%d] %s is an acceptable macro call',
-                    #     $rc{pos}, $rc{pos}, $maxPos, $lexemeValue );
-                    $self->logger_debug( '[%d..%d/%d] %s ...',
-                        $rc{pos}, $rc{pos}, $maxPos, $printableMacroName );
-
-                    if ( $lparenPos < 0 ) {
-                        #
-                        # Execute the macro without argument
-                        #
-                        $self->logger_debug( '[%d..%d/%d] %s -> ???',
-                            $rc{pos}, $rc{pos}, $maxPos,
-                            $printableMacroName );
-                        $lexemeValue = $thisMacro->macro_execute($self);
-                        if ( length($lexemeValue) > 0 ) {
-                            $self->logger_debug(
-                                '[%d..%d/%d] %s -> %s',
-                                $rc{pos},
-                                $rc{pos},
-                                $maxPos,
-                                $printableMacroName,
-                                $self->_printable($lexemeValue)
-                            );
-                        }
-                        else {
-                            $self->logger_debug( '[%d..%d/%d] %s',
-                                $rc{pos}, $rc{pos}, $maxPos,
-                                $printableMacroName );
-                        }
-                    }
-                    else {
-                        #
-                        # Call us recursively.
-                        # This will change $lexemeValue and next position
-                        #
-                        my $afterRparenPos;
                         $self->_set__parse_level( $self->_parse_level + 1 );
                         my $dict = $self->_parseByGrammar(
                             $inputRef,           $lparenPos,
                             $BYMACROARGUMENTS_G, $thisMacro
                         );
                         $self->_set__parse_level( $self->_parse_level - 1 );
-
-                        # $self->logger_debug( '[%d..%d/%d] <= %s',
-                        #     $rc{pos}, $rc{pos}, $maxPos, $dict );
-
-                        my $parametersValue = $dict->{value};
-                        $afterRparenPos = $dict->{pos};
-                        #
-                        # Execute the macro
-                        #
-                        my $printableArguments = join( ', ',
-                            map { $self->_printable($_) }
-                                $parametersValue->value_elements );
-                        $self->logger_debug(
-                            '[%d..%d/%d] %s(%s) -> ???',
-                            $rc{pos},
-                            $rc{pos},
-                            $maxPos,
-                            $printableMacroName,
-                            $printableArguments
-                        );
-                        $lexemeValue = $thisMacro->macro_execute( $self,
-                            $parametersValue->value_elements );
-                        if ( length($lexemeValue) > 0 ) {
-                            $self->logger_debug(
-                                '[%d..%d/%d] %s(...) -> %s',
-                                $rc{pos},
-                                $rc{pos},
-                                $maxPos,
-                                $printableMacroName,
-                                $self->_printable($lexemeValue)
-                            );
+                        if ( Undef->check($dict) ) {
+                            goto return_error;
                         }
-                        else {
-                            $self->logger_debug( '[%d..%d/%d] %s(...)',
-                                $rc{pos}, $rc{pos}, $maxPos,
-                                $printableMacroName );
-                        }
-                        $lexemeLength = $afterRparenPos - $rc{pos};
+                        @args         = $dict->{value}->value_elements;
+                        $lexemeLength = $dict->{pos} - $rc{pos};
                     }
                     #
+                    # It is the reponsability of implementation to make sure
+                    # that a macro never croaks.
+                    #
+                    $lexemeValue
+                        = $self->impl_macroExecute( $thisMacro, @args );
+                    #
                     # Eventual postmatch length
                     #
                     $lexemeLength
@@ -460,12 +410,6 @@ COMMA ~ ',' _WS_any
                         #     $rc{pos}, $rc{pos}, $maxPos, ${$inputRef} );
 
                         $maxPos = length( ${$inputRef} ) - 1;
-                        #
-                        # Protect the case of empty string
-                        #
-                        if ( $rc{pos} > $maxPos ) {
-                            return undef;
-                        }
                         goto again;
                     }
                 }
@@ -508,6 +452,9 @@ COMMA ~ ',' _WS_any
                 #
             }
             else {
+                #
+                # We are at the top level: flush to current diversion
+                #
                 my $tmpValue = MarpaX::Languages::M4::Impl::Value->new()
                     ->value_push($lexemeValue);
                 $self->impl_appendValue(
@@ -524,10 +471,8 @@ COMMA ~ ',' _WS_any
             #
             local $MarpaX::Languages::M4::Impl::Parser::macro = $macro;
             my $valueRef = $r->value;
-            if ( Undef->check($valueRef) ) {
-                NoParseValue->throw(
-                    "No parse value starting at position $pos (20 first characters): "
-                        . substr( ${$inputRef}, $pos, 20 ) );
+            if ( Undef->check($valueRef) || Undef->check( ${$valueRef} ) ) {
+                goto return_error;
             }
 
             $rc{value} = ${$valueRef};
@@ -543,6 +488,23 @@ COMMA ~ ',' _WS_any
         #     $rc{pos}, $rc{pos}, $maxPos, \%rc );
 
         return \%rc;
+
+    return_error:
+        #
+        # We propagate the undef to all levels except number 0
+        #
+        if ( $self->_parse_level > 0 ) {
+            if ( $self->_parse_level == 1 && $self->impl_eoi ) {
+                #
+                # We want the parsing to stop now
+                #
+                $self->impl_raiseException('EOF during argument collection');
+            }
+            return;
+        }
+        else {
+            return \%rc;
+        }
     }
 
     #
@@ -559,19 +521,6 @@ COMMA ~ ',' _WS_any
         return ${$inputRef};
     }
 
-    method _printable (Str|M4Macro $input, Bool $noQuote? --> Str) {
-        $noQuote //= false;
-        #
-        # If M4Macro let's get the object representation stringified
-        #
-        my $printable = Str->check($input) ? $input : "$input";
-
-        # $printable =~ s/([^[:print:]])/sprintf('0x%x', ord($1))/eg;
-        return Str->check($input)
-            ? ( $noQuote ? $printable : $self->impl_quote($printable) )
-            : $printable;
-    }
-
 }
 
 1;
@@ -588,7 +537,7 @@ MarpaX::Languages::M4::Impl::Parser - M4 Marpa parser
 
 =head1 VERSION
 
-version 0.006
+version 0.007
 
 =head1 AUTHOR
 
