@@ -6,7 +6,7 @@ use Moops;
 
 role MarpaX::Languages::M4::Role::Impl {
 
-    our $VERSION = '0.006'; # VERSION
+    our $VERSION = '0.007'; # VERSION
 
     our $AUTHORITY = 'cpan:JDDPAUSE'; # AUTHORITY
 
@@ -23,9 +23,15 @@ role MarpaX::Languages::M4::Role::Impl {
     requires 'impl_parse';
     requires 'impl_unparsed';
     requires 'impl_setEoi';
+    requires 'impl_eoi';
+    requires 'impl_raiseException';
     requires 'impl_line';
     requires 'impl_debugfile';
     requires 'impl_rc';
+    requires 'impl_isImplException';
+    requires 'impl_macroExecute';
+    requires 'impl_macroCallId';
+    requires 'impl_debugfile';
 
     with 'MarpaX::Languages::M4::Role::Builtin';
     with 'MarpaX::Languages::M4::Role::Logger';
@@ -46,26 +52,7 @@ MarpaX::Languages::M4::Role::Impl - M4 implementation role
 
 =head1 VERSION
 
-version 0.006
-
-=for :stopwords cpan testmatrix url annocpan anno bugtracker rt cpants kwalitee diff irc mailto metadata placeholders metacpan
-
-=head1 SUPPORT
-
-=head2 Bugs / Feature Requests
-
-Please report any bugs or feature requests through the issue tracker
-at L<https://rt.cpan.org/Public/Dist/Display.html?Name=MarpaX-Languages-M4>.
-You will be notified automatically of any progress on your issue.
-
-=head2 Source Code
-
-This is open source software.  The code repository is available for
-public review and contribution under the terms of the license.
-
-L<https://github.com/jddurand/marpax-languages-m4>
-
-  git clone git://github.com/jddurand/marpax-languages-m4.git
+version 0.007
 
 =head1 AUTHOR
 
